﻿using System;
using System.Diagnostics;
using System.Globalization;
using System.Runtime.InteropServices;
using System.ComponentModel.Design;
using Microsoft.Win32;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.OLE.Interop;
using Microsoft.VisualStudio.Shell;
using EnvDTE80;
using EnvDTE;
using EnvDTE90;

namespace VitalyZayko.Code4Blog
{
    /// <summary>
    /// This is the class that implements the package exposed by this assembly.
    ///
    /// The minimum requirement for a class to be considered a valid package for Visual Studio
    /// is to implement the IVsPackage interface and register itself with the shell.
    /// This package uses the helper classes defined inside the Managed Package Framework (MPF)
    /// to do it: it derives from the Package class that provides the implementation of the 
    /// IVsPackage interface and uses the registration attributes defined in the framework to 
    /// register itself and its components with the shell.
    /// </summary>
    // This attribute tells the PkgDef creation utility (CreatePkgDef.exe) that this class is
    // a package.
    [PackageRegistration(UseManagedResourcesOnly = true)]
    // This attribute is used to register the informations needed to show the this package
    // in the Help/About dialog of Visual Studio.
    [InstalledProductRegistration("#110", "#112", "1.2", IconResourceID = 400)]
    // This attribute is needed to let the shell know that this package exposes some menus.
    [ProvideMenuResource("Menus.ctmenu", 1)]
    [Guid(GuidList.guidCode4BlogPkgString)]
    [ProvideOptionPage(typeof(Code4BlogOptions), "Code4Blog", "General", 0, 0, true)]
    public sealed class Code4BlogPackage : Package
    {
        private DTE2 dte;

        /// <summary>
        /// Default constructor of the package.
        /// Inside this method you can place any initialization code that does not require 
        /// any Visual Studio service because at this point the package object is created but 
        /// not sited yet inside Visual Studio environment. The place to do all the other 
        /// initialization is the Initialize method.
        /// </summary>
        public Code4BlogPackage()
        {
            Trace.WriteLine(string.Format(CultureInfo.CurrentCulture, "Entering constructor for: {0}", this.ToString()));
        }



        /////////////////////////////////////////////////////////////////////////////
        // Overriden Package Implementation
        #region Package Members

        /// <summary>
        /// Initialization of the package; this method is called right after the package is sited, so this is the place
        /// where you can put all the initilaization code that rely on services provided by VisualStudio.
        /// </summary>
        protected override void Initialize()
        {
            Trace.WriteLine (string.Format(CultureInfo.CurrentCulture, "Entering Initialize() of: {0}", this.ToString()));
            base.Initialize();

            // Add our command handlers for menu (commands must exist in the .vsct file)
            OleMenuCommandService mcs = GetService(typeof(IMenuCommandService)) as OleMenuCommandService;
            if ( null != mcs )
            {
                // Create the command for the menu item.
                CommandID menuCommandID = new CommandID(GuidList.guidCode4BlogCmdSet, (int)PkgCmdIDList.cmdidCode4BlogMenu);
                MenuCommand menuItem = new MenuCommand(MenuItemCallback, menuCommandID );
                mcs.AddCommand( menuItem );
            }

            dte = (DTE2)GetService(typeof(DTE));
        }
        #endregion

        /// <summary>
        /// This function is the callback used to execute a command when the a menu item is clicked.
        /// See the Initialize method to see how the menu item is associated to this function using
        /// the OleMenuCommandService service and the MenuCommand class.
        /// </summary>
        private void MenuItemCallback(object sender, EventArgs e)
        {
            if (dte == null)
            {
                Trace.WriteLine("DTE2 is null");
                ShowMessage("Unable to initialize DRE2!");
                return;
            }
            
            if (dte.ActiveDocument == null)
            {
                ShowMessage("No Active Document detected! Please open a Document and click on its surface to activate.");
                Trace.WriteLine("DTE.ActiveDocument is null");
                return;
            }
            
            ProcessCode();
        }

        /// <summary>
        /// Showing messages
        /// </summary>
        /// <param name="Msg">A staring to show</param>
        private void ShowMessage(string Msg)
        {
            IVsUIShell uiShell = (IVsUIShell)GetService(typeof(SVsUIShell));
            Guid clsid = Guid.Empty;
            int result;
            Microsoft.VisualStudio.ErrorHandler.ThrowOnFailure(uiShell.ShowMessageBox(
                       0,
                       ref clsid,
                       "Code4Blog",
                       Msg,
                       string.Empty,
                       0,
                       OLEMSGBUTTON.OLEMSGBUTTON_OK,
                       OLEMSGDEFBUTTON.OLEMSGDEFBUTTON_FIRST,
                       OLEMSGICON.OLEMSGICON_CRITICAL,
                       0,        // false
                       out result));
        }

        /// <summary>
        /// Main process
        /// </summary>
        private void ProcessCode()
        {
            TextDocument doc = (TextDocument)(dte.ActiveDocument.Object("TextDocument"));

            dte.StatusBar.Text = String.Format("Converting \"{0}\" to HTML... Please wait.", dte.ActiveDocument.Name);
            dte.StatusBar.Animate(true, vsStatusAnimation.vsStatusAnimationDeploy);

            Code4BlogOptions opt = (Code4BlogOptions)GetDialogPage(typeof(Code4BlogOptions));
            EnvDTE.Properties props = dte.get_Properties("TextEditor", "HTML");
            int optValue = (int)props.Item("IndentStyle").Value;

            try
            {
                if (doc.Selection.IsEmpty)
                {
                    doc.Selection.SelectAll();
                    doc.Selection.Copy();
                    doc.Selection.StartOfLine();
                }
                else
                    doc.Selection.Copy();

                // Set Option value to zero (None):
                props.Item("IndentStyle").Value = 0;

                HtmlConverter converter = new HtmlConverter(opt.EvenColor, opt.OddColor, opt.DrawBorder, opt.LineNumbers, opt.MaxWidth, opt.MaxHeight, opt.FontFamily, opt.FontSize);

                Window htmlWnd = dte.ItemOperations.NewFile(@"General\HTML Page", "Code4Blog");

                TextDocument htmlDoc = (TextDocument)htmlWnd.Document.Object("TextDocument");
                htmlDoc.Selection.SelectAll();

                EditPoint ep = htmlDoc.StartPoint.CreateEditPoint();
                ep.StartOfDocument();
                ep.ReplaceText(htmlDoc.Selection.BottomPoint, converter.GetHtmlFromClipboard(), (int)vsEPReplaceTextOptions.vsEPReplaceTextTabsSpaces);

                HTMLWindow3 html = (HTMLWindow3)htmlWnd.Object;
                html.CurrentPane = vsHTMLPanes.vsHTMLPaneSplit;
                html.CurrentView = vsHTMLViews.vsHTMLViewSource;

                htmlDoc.Selection.SelectAll();
            }
            catch (Exception ex)
            {
                ShowMessage("Unsupported source format! Cannot convert specified code.\r\nException details: " + ex.Message);
            }
            finally
            {
                opt.Dispose();
                // Restore Option value:
                props.Item("IndentStyle").Value = optValue;
                dte.StatusBar.Text = String.Empty;
                dte.StatusBar.Animate(false, vsStatusAnimation.vsStatusAnimationDeploy);
            }
        }

    }
}


